# -*- coding: utf-8 -*-
"""
ICSynthFunctions.py - functions for interacting with ICSynth light

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2018
stardrop-support@optibrium.com
"""

import ICSynthWebServices
import PythonQt
import webbrowser

#maximum number of molecules user can request to query at once
SINGLE_REQUEST_MOLECULE_LIMIT = 1


def query_selected(stardrop):
    """
    Get SMILES strings for molecules in currently selected rows
    Send these to ICSynth light REST API and open the URL(s) we get back in a browser
    
    :param stardrop:    stardrop appui object
    :return: None
    """
    currentSelected = []
    if SINGLE_REQUEST_MOLECULE_LIMIT > 1:
        currentSelected = stardrop.getCurrentMolecules()
    else:
        currentMolecule = stardrop.getCurrentMolecule()
        if currentMolecule:
            currentSelected = [currentMolecule]
            
    if not currentSelected:
        stardrop.show('No molecule selected for ICSynth light request.')
        return
    query_molecules(stardrop, currentSelected)
    
    
def query_molecules(stardrop, molecules):
    """
    Send the given SMILES string to ICSynth light REST API and open the URL(s) we get back in a browser.
    Shows a progress dialog while the requests are being processed
    
    :param stardrop:    stardrop appui object
    :param molecules:   list of SMILES strings to query
    :return: None
    """
    requestsToMake = min(len(molecules), SINGLE_REQUEST_MOLECULE_LIMIT)
    if requestsToMake < 1:
        return
    
    progDlg = PythonQt.SimpleProgress()
    progDlg.modal = True
    progDlg.setWindowTitle('ICSynth light')
    progDlg.setMessage("Sending " + str(requestsToMake) + " request(s)")
    progDlg.setMaximum(requestsToMake)
    progDlg.setProgress(0)
    progDlg.show()
    stardrop.updateUI()
    
    for idx, smiles in enumerate(molecules):
        if idx >= SINGLE_REQUEST_MOLECULE_LIMIT:
            break
        if not query_SMILES_and_open_URL(smiles):
            stardrop.show('Request was not successful.')
        progDlg.setProgress(idx + 1)
        stardrop.updateUI()
    progDlg.hide()
    progDlg.delete()
    

def query_SMILES_and_open_URL(SMILES):
    """
    Send molecule smiles to ICSynth light REST API and open the URL we get back in a browser
    
    :param SMILES:    SMILES string to query
    :return:    Whether an URL was successfuly opened in a browser
    """
    URLtoOpen = ICSynthWebServices.query_SMILES(SMILES)
        
    if not URLtoOpen:
        return False
        
    try:
        webbrowser.open(URLtoOpen)
        return True
    except Exception:
        return False
        
def open_about_dialog(stardrop):
    """
    Opens an about dialog with information about ICSynth light
    
    :param stardrop:    stardrop appui object
    :return: None
    """
    stardrop.show('ICSYNTH light<br><br>'
                'Version 0.1 (August 2018)<br><br>'
                'This application is a light version of InfoChem’s synthesis planning tool <a href="http://www.infochem.de/products/software/icsynth.shtml">ICSYNTH</a>,'
'                a powerful computer aided synthesis design system that enables chemists to generate synthetic pathways for a target molecule.<br><br>'
                'For info, email: <a href="mailto:info@infochem.de">info@infochem.de</a><br><br>'
                'Copyright © 2018 InfoChem GmbH. All Rights Reserved.<br><br>'
                '<a href="http://www.infochem.de/">www.infochem.de</a>')
    
        
        
def get_stardrop_definitions():
    definitions = [
        {
            'script_name': 'ICSynth light/Synthetic assessment...',
            'callback': query_selected
        },
        {
            'script_name': 'ICSynth light/About',
            'callback': open_about_dialog
        }
    ]
    
    return definitions