"""
ICSynthWebServices.py - functions that query ICSynth light web service

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2018
stardrop-support@optibrium.com
"""


import json
import requests
import time
import os
import datetime


class Logger:
    def __init__(self,filename, purge = True):
        self.filename = filename
        if purge or not os.path.isfile(self.filename):
            fout = open(self.filename,'w')
            fout.close()
        
    def log(self,message):
        fout = open(self.filename,'a')
        dt = datetime.datetime.now()
        fout.write(dt.strftime('%d/%m/%Y %H:%M ') + message + '\n')
        fout.close()

log = Logger('ICSynth.log')

#global storage of login token
ICSynthLoginToken = ""

#base URL for login and query requests
BASE_URL = "https://icsynth-light.infochem.de/"

TOKEN_FILE_PATH = "icsltoken.txt"
        
        
def query_SMILES(SMILES):
    """
    Send molecule smiles to ICSynth light REST API and return an URL to open in a browser
    
    :param SMILES:  SMILES string to query
    :return:    string containing the URL returned by the query request
    """
    log.log('query_SMILES: ' + SMILES)
    
    loginToken = get_login_token()
    
    URLtoOpen = make_query_request(SMILES, loginToken)

    return URLtoOpen


def get_login_token(forceRelog = False):
    """
    Get a login token, either an existing one or a new one from a login request
    The token is stored globally and this function will also update the global variable
    
    :param forceRelog:    whether to login to get a new token even if one already exists
    :return: login token
    """
    global ICSynthLoginToken
    if not forceRelog and not ICSynthLoginToken:
        ICSynthLoginToken = retrieve_login_token()
    
    if forceRelog or not ICSynthLoginToken:
        ICSynthLoginToken = login('optibrium', 'RX961ium20')
        if ICSynthLoginToken:
            save_login_token(ICSynthLoginToken)
        
    return ICSynthLoginToken
    
def retrieve_login_token():
    """
    Attempt to read a saved token.
    :return: A string token if available, otherwise an empty string
    """
    try:
        file = open(TOKEN_FILE_PATH)
        token = file.read()
        file.close()
        return token
    except:
        pass
    return ''
    
def save_login_token(token):
    """
    Save a token for future use.
    :param token: The new token
    """
    try:
        file = open(TOKEN_FILE_PATH,'w')
        file.write(token)
        file.close()
    except:
        pass


def login(username, password):
    """
    Login to the ICSynth light service and return a login token.
    
    :param username:    username to use for login
    :param password:    password to use for login
    :return: login token, or empty string if the login was not successful
    """
    loginData = {"username": username, "password": password}
            
    loginURL = BASE_URL + 'login'
    log.log(loginURL)
    headers = {'content-type': 'application/json'}
    
    try:
        response = requests.post(loginURL, data=json.dumps(loginData), headers=headers, allow_redirects=True)
        log.log('loginResponse: ' + str(response.status_code))
        if response.status_code == 200:
            return response.headers['Authorization']
        
        return ""
    except Exception as e:
        log.log(repr(e))
        return ""
        
        
def make_query_request(SMILES, loginToken, retryLogin = True):
    """
    Send molecule smiles with an authorization token to ICSynth light REST API and return an URL to open in a browser
    
    :param SMILES:      SMILES string to query
    :param loginToken:  login token used for authorization
    :param retryLogin:  whether to try logining in again if the token has expired
    :return:    string containing the URL returned by the query request
    """
    data = {'format': 'SMILES', 'structure': SMILES}

    queryURL = BASE_URL + 'api/query'
    log.log(queryURL)
    authHeader = loginToken
    headers = {
            'content-type': 'application/json',
            'Authorization': authHeader
        }
    
    try:
        response = requests.post(queryURL, data=json.dumps(data), headers=headers, allow_redirects=True)
        log.log('response: ' + str(response.status_code))
        
        #try again with a new login token if the current one expired
        if retryLogin and does_response_have_expired_token_exception(response):
            loginToken = get_login_token(True)
            return make_query_request(SMILES, loginToken, False)      
        
        URLtoOpen = get_value_from_json_response(response,'url')
        return URLtoOpen
    except Exception as e:
        log.log(repr(e))
        return ""
        

def does_response_have_expired_token_exception(response):
    """
    Returns whether the given response from ICSynth light REST API has raised an expired token exception
    Note: A 401 response indicates an expired token, but to be extra safe we also check that the response has a json format
    """
    if response.status_code == 401:
        return True
    
    try:
        response.json()
    except Exception as e:
        log.log(repr(e))
        return True
             
    return False
    
def get_value_from_json_response(response, param):
    """
    Returns a value for the given parameter from the given response from ICSynth light REST API.
    Response is expected to be in a JSON format
    """
    try:
        responseJSON = response.json()
        return responseJSON[param]
    except Exception as e:
        log.log(repr(e))
        return ""